; ECHO.SYS - echo device driver for M$ DOS
; Copyright (c) 2000 Andreas Buening
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; Compiling the driver:
; tasm /la /t /m /zn echo.asm
; tlink echo.obj
; exe2bin echo.exe
; ren echo.bin echo.sys
;
; Usage:
; DEVICE=...\echo.sys text
; Displays text on stdout. There are two special characters: '/' and '\'
; '/' converts a uppercase letter to a lowercase (note: in M$ DOS's config.sys
;     only uppercase letters are allowed.)
; '\' generates a Control character: \e or \E generates the Escape character (ascii 27)
;
; To set the foreground color to red and the backgroud color to blue you can use
; DEVICE=...\echo.sys \e[0;31;44/m
; in your config.sys. This will produce the ANSI escape sequence ESC[0;31;44m
;


IDEAL
JUMPS
LOCALS

;========================================================= CONSTANTS
; Status codes
stSuccess 	EQU	0100h
stError		EQU	8100h
stUnknownCmd	EQU	8103h
stDrvNotReady	EQU	8102h

MediaID		EQU	0F0h	; Media ID byte - "other"

;========================================================= Driver segment
SEGMENT DrvSeg
	ASSUME cs:DrvSeg,ds:NOTHING,es:NOTHING,ss:NOTHING
; Device header
	DW	0FFFFh, 0	; Link to next driver
	DW	0h		; Attributes
	DW	Strategy, Interrupt
	DB	1, 7 DUP (0)	; Number of units

;========================================================= Resident DATA area
; Data area
ReqHdrOfs DW	0		; Offset ot request header
ReqHdrSeg DW	0		; Segment of request header

;========================================================= STRATEGY routine
PROC	Strategy FAR
	mov	[ReqHdrOfs], bx
	mov	[ReqHdrSeg], es
	ret
ENDP	Strategy

;========================================================= INTERRUPT routine
PROC	Interrupt FAR
	push	ax bx cx dx si di bp ds es
	pushf

	push	cs
	pop	ds
	ASSUME	ds:DrvSeg
	les	bx, [DWORD ReqHdrOfs]
	mov	al, [es:bx+2]	; Function number

	cmp	al, 00h
        jne	@@errUnknownCommand
        call	Init
	jmp	@@Exit

@@errUnknownCommand:
	mov	[WORD es:bx+3], stUnknownCmd
@@Exit:	popf
	pop	es ds bp di si dx cx bx ax
	ret
ENDP	Interrupt

;========================================================= INIT Code
PROC	Init
	call DisplayString
	mov    	ah, 09h
        mov     dx, OFFSET EndLine
        int	21h

	mov	[BYTE es:bx+13], 0    ; 0 drives supported
	mov	[WORD es:bx+14], 0    ; remove the whole driver from memory
	mov	[WORD es:bx+3], stError ; error while executing function

	ret
ENDP	Init
        
;---------------------------------------------------------------------
; Display command line parameters
PROC 	DisplayString
	push	ds
	push	bx cx dx si di
	cld
	lds	si, [es:bx+18]
	ASSUME	ds:NOTHING
; Scan past the filename to the first space
@@EatFilename:
	lodsb
	cmp	al, 0Dh
	je	@@Exit
	cmp	al, 0Ah
	je	@@Exit
	cmp	al, ' '
	jne	@@EatFilename

@@OneChar:   				; '/' and '\' are special charakters
	lodsb
        cmp	al, '\'
        je	@@BackSlash
        cmp	al, '/'
        je	@@Slash
	cmp	al, 0Dh
        je	@@Finish
        cmp	al, 0Ah
        je	@@Finish
        
@@DisplayChar:				; Displays the byte in al
	mov	dl, al
        mov	ah, 02h
        int	21h
	jmp	@@OneChar
        
@@BackSlash:                    	; if the next byte is 'E' or 'e' display ESC (27)
	lodsb
        cmp	al, 'E'
        je	@@Escape
        cmp	al, 'e'
        je	@@Escape
        jmp 	@@DisplayChar
        
@@Slash:				; if the next byte is A..Z, display a..z
	lodsb
        cmp	al, 'A'
        jb      @@DisplayChar
        cmp	al, 'Z'
        ja	@@DisplayChar
        add	al, 032
        jmp	@@DisplayChar

@@Escape:
	mov	al, 027
        jmp 	@@DisplayChar
        
@@Finish:
@@Exit:
        pop	di si dx cx bx
	pop	ds
	ret
ENDP    DisplayString


;========================================================= INIT data
EndLine DB 10,13,"$"

ENDS	DrvSeg
	END
